import argparse
import json
import pathlib
import sys

import yaml
import jiml

FORMATS = ['json', 'yaml']

LOADERS = {
    'json': json.loads,
    'yaml': yaml.safe_load,
}

DUMPERS = {
    'json': lambda *args, allow_unicode=False, **kwargs: json.dumps(
        *args, ensure_ascii=not allow_unicode, **kwargs
    ),
    'yaml': yaml.dump,
}


def parse_args():
    parser = argparse.ArgumentParser(description='Convert json to json.')
    parser.add_argument(
        '-t', '--template',
        required=True,
        type=pathlib.Path,
        help='jinja template path'
    )
    parser.add_argument(
        '-i', '--input',
        type=pathlib.Path,
        help='input json path (stdin by default)'
    )
    parser.add_argument(
        '-o', '--output',
        type=pathlib.Path,
        help='output json path (stdout by default)'
    )

    parser.add_argument(
        '-I', '--input-format',
        choices=FORMATS,
    )
    parser.add_argument(
        '-O', '--output-format',
        choices=FORMATS,
    )

    parser.add_argument('-s', '--sort-keys', action='store_true')
    parser.add_argument('-u', '--allow-unicode', action='store_true')
    parser.add_argument('-n', '--indent', type=int)

    parser.add_argument('-A', '--no-autoescape', action='store_true')

    return parser.parse_args()


def main():
    args = parse_args()
    template = args.template.read_text()
    if args.input is None:
        input_text = sys.stdin.read()
        input_suffix = None
    else:
        input_text = args.input.read_text()
        input_suffix = args.input.suffix.strip('.')
        input_suffix = input_suffix if input_suffix in FORMATS else None

    if args.output is None:
        output_suffix = None
    else:
        output_suffix = args.output.suffix.strip('.')
        output_suffix = output_suffix if output_suffix in FORMATS else None

    input_format = args.input_format or input_suffix or 'json'
    output_format = args.output_format or output_suffix or 'json'

    input_data = LOADERS[input_format](input_text)
    output_data = jiml.convert(template, input_data, autoescape=not args.no_autoescape)
    output_text = DUMPERS[output_format](
        output_data,
        indent=args.indent,
        allow_unicode=args.allow_unicode,
        sort_keys=args.sort_keys,
    )

    if args.output is None:
        sys.stdout.write(output_text)
    else:
        args.output.write_text(output_text)


if __name__ == '__main__':
    main()



import json
import pathlib

import jinja2
import yaml
from jinja_vanish import DynAutoEscapeEnvironment, markup_escape_func


__all__ = (
    'qstr', 'str', 'json_dumps',
    'FILTERS',
    'config',
    'JimlTemplate', 'JimlEnvironment',
    'load_template', 'render', 'convert',
)


# TODO: add exceptions


def yaml_escape(obj):
    if isinstance(obj, jinja2.Undefined):
        if isinstance(obj, jinja2.StrictUndefined):
            raise jinja2.exceptions.UndefinedError(f"Can't serialize StrictUndefined var '{obj._undefined_name}'")
        return ''

    res = yaml.dump(
        obj,
        width=float("inf"),
        default_flow_style=True,
        default_style='"',
    ).strip('\n')

    if '\n' in res:
        raise ValueError('escaped value contains newline symbol for object {}'.format(obj))

    return res


def _qstr(inp):
    if inp is None:
        inp = ''
    return json.dumps(str(inp))


def qstr(inp):
    return jinja2.Markup(_qstr(inp))


def str_(inp):
    return jinja2.Markup(_qstr(inp)[1:-1])


def json_dumps(inp):
    return jinja2.Markup(json.dumps(inp, ensure_ascii=False))


JIML_FILTERS = {
    'qstr': qstr,
    'str': str_,
    'json.dumps': json_dumps,
    'yaml.dumps': yaml_escape,
    'e': yaml_escape,
    'escape': yaml_escape,
}


class JimlTemplate(jinja2.Template):
    def convert(self, *args, **kwargs):
        return yaml.safe_load(self.render(*args, **kwargs))

    __call__ = convert


class JimlEnvironment(DynAutoEscapeEnvironment):
    template_class = JimlTemplate

    def __init__(self, *args, autoescape=False, escape_func=None, **kwargs):
        if autoescape and escape_func is None:
            escape_func = markup_escape_func(yaml_escape)
        super().__init__(*args, autoescape=autoescape, escape_func=escape_func, **kwargs)
        self.filters.update(JIML_FILTERS)


_env = None


class EnvOptions:
    def __init__(self):
        self.values = {}

    def update(self, values):
        global _env

        self.values.update(values)
        _env = JimlEnvironment(**self.values)


config = EnvOptions()
config.update({
    'autoescape': True,
    'undefined': jinja2.StrictUndefined,
})
assert _env is not None


def load_template(template=None, path=None, env=None):
    if template is None:
        template = pathlib.Path(path).read_text()
    env = env or _env
    return env.from_string(template)


def render(template, context, env=None):
    return load_template(template, env=env).render(context)


def convert(template, context, env=None):
    return load_template(template, env=env).convert(context)



from distutils.core import setup
from pathlib import Path

this_directory = Path(__file__).parent

long_description = (this_directory / "README.md").read_text()


setup(
    name='jiml',
    packages=['jiml'],
    version='0.1.3',
    license='MIT',
    description='Convert json to json using jinja2 and yaml',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Vladimir',
    author_email='vmstarchenko@edu.hse.ru',
    url='https://github.com/vmstarchenko/jiml',
    keywords=['json2json', 'json', 'yaml', 'template'],
    install_requires=[
        'PyYAML',
        'Jinja2',
        'jinja-vanish',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    zip_safe=True,
)
